import { idGenerator } from '../id-generator';

const nextId = idGenerator();

const requestAnimationFrame = (
  window.requestAnimationFrame || /* istanbul ignore next */
  window.webkitRequestAnimationFrame || /* istanbul ignore next */
  window.mozRequestAnimationFrame || /* istanbul ignore next */
  window.oRequestAnimationFrame || /* istanbul ignore next */
  window.msRequestAnimationFrame
);

const cancelAnimationFrame = (
  window.cancelAnimationFrame || /* istanbul ignore next */
  window.webkitCancelRequestAnimationFrame || /* istanbul ignore next */
  window.webkitCancelAnimationFrame || /* istanbul ignore next */
  window.mozCancelRequestAnimationFrame || /* istanbul ignore next */
  window.mozCancelAnimationFrame || /* istanbul ignore next */
  window.oCancelRequestAnimationFrame || /* istanbul ignore next */
  window.oCancelAnimationFrame || /* istanbul ignore next */
  window.msCancelRequestAnimationFrame || /* istanbul ignore next */
  window.msCancelAnimationFrame
);

export class Async {
  static nextFrame (callback) {
    return requestAnimationFrame(callback);
  }

  static sleep (wait) {
    return new Promise(resolve => {
      setTimeout(() => this.nextFrame(resolve), wait);
    });
  }

  static run (callback, wait) {
    const async = new Async();
    async.start(callback, wait);
    return async;
  }

  static cancel (async) {
    if (async && typeof async.cancel === 'function') {
      async.cancel();
    }
  }

  constructor (context) {
    this.id = nextId();
    this.context = context;
    // this.handle = null;
    // this.frameHandle = null;
    this.cleared = true;
  }

  start (callback, wait) {
    if (typeof callback !== 'function') {
      throw new Error('Async should specify function');
    }

    if (!this.cleared) {
      throw new Error('Async already run');
    }

    this.cleared = false;

    const self = this;
    const context = this.context;
    const boundCallback = function () {
      self.frameHandle = requestAnimationFrame(() => {
        self.__clear();
        callback.call(context);
      });
    };

    if (wait) {
      this.handle = setTimeout(boundCallback, wait);
    } else {
      boundCallback();
    }
  }

  __clear () {
    this.cleared = true;

    cancelAnimationFrame(~~this.frameHandle);
    clearTimeout(~~this.handle);
    this.handle = this.frameHandle = undefined;
  }

  cancel () {
    this.__clear();
  }
}
